package com.microduino.flutter_js_bridge;

import android.annotation.TargetApi;
import android.os.Build;
import android.webkit.WebResourceRequest;
import android.webkit.WebView;

import com.github.lzyzsd.jsbridge.BridgeWebView;
import com.github.lzyzsd.jsbridge.BridgeWebViewClient;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

/**
 * Created by Gloria Guo on ${Date}.
 */
public class FlutterBridgeWebViewClient extends BridgeWebViewClient {
    private final MethodChannel methodChannel;
    private boolean hasNavigationDelegate;

    public FlutterBridgeWebViewClient(BridgeWebView webview, MethodChannel methodChannel) {
        super(webview);
        this.methodChannel = methodChannel;
    }

    void setHasNavigationDelegate(boolean hasNavigationDelegate) {
        this.hasNavigationDelegate = hasNavigationDelegate;
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    @Override
    public boolean shouldOverrideUrlLoading(WebView view, WebResourceRequest request) {
        if (!hasNavigationDelegate) {
            return false;
        }
        notifyOnNavigationRequest(
                request.getUrl().toString(), request.getRequestHeaders(), view, request.isForMainFrame());
        return request.isForMainFrame();
    }

    @Override
    public boolean shouldOverrideUrlLoading(WebView view, String url) {
        if (!hasNavigationDelegate) {
            return false;
        }
        notifyOnNavigationRequest(url, null, view, true);
        return true;
    }

    @Override
    public void onPageFinished(WebView view, String url) {
        Map<String, Object> args = new HashMap<>();
        args.put("url", url);
        methodChannel.invokeMethod("onPageFinished", args);
    }

    private void notifyOnNavigationRequest(
            String url, Map<String, String> headers, WebView webview, boolean isMainFrame) {
        HashMap<String, Object> args = new HashMap<>();
        args.put("url", url);
        args.put("isForMainFrame", isMainFrame);
        if (isMainFrame) {
            methodChannel.invokeMethod(
                    "navigationRequest", args, new OnNavigationRequestResult(url, headers, webview));
        } else {
            methodChannel.invokeMethod("navigationRequest", args);
        }
    }

    private static class OnNavigationRequestResult implements MethodChannel.Result {
        private final String url;
        private final Map<String, String> headers;
        private final WebView webView;

        private OnNavigationRequestResult(String url, Map<String, String> headers, WebView webView) {
            this.url = url;
            this.headers = headers;
            this.webView = webView;
        }

        @Override
        public void success(Object shouldLoad) {
            Boolean typedShouldLoad = (Boolean) shouldLoad;
            if (typedShouldLoad) {
                loadUrl();
            }
        }

        @Override
        public void error(String errorCode, String s1, Object o) {
            throw new IllegalStateException("navigationRequest calls must succeed");
        }

        @Override
        public void notImplemented() {
            throw new IllegalStateException(
                    "navigationRequest must be implemented by the webview method channel");
        }

        private void loadUrl() {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                webView.loadUrl(url, headers);
            } else {
                webView.loadUrl(url);
            }
        }
    }
}
